// Copyright (C) 2006, 2007, 2008, 2009 by Florent Lamiraux, Thomas
// Moulard, Mathieu Poirier, CNRS.
//
// This file is part of the hppWalkPlanner.
//
// hppWalkPlanner is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.
//
// hppWalkPlanner is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with hppWalkPlanner. If not, see
// <http://www.gnu.org/licenses/>.

#include "kdunix-warning.hh"

#include <iostream>
#include <fstream>

#include <jrlMathTools/angle.h>

#include <kwsIO/kwsioConfig.h>

#include <KineoKCDModel/kppKCDBox.h>
#include <KineoModel/kppFreeFlyerJointComponent.h>
#include <KineoModel/kppSolidComponent.h>
#include <KineoModel/kppSolidComponentRef.h>
#include <KineoWorks2/kwsSteeringMethod.h>

#include <hpp/util/debug.hh>

#include "test-directpath.hh"

ktStatus
TestDirectPath::init ()
{
  if (createDevice() != KD_OK)
    {
      hppDout (error, "failed to create device");
      return KD_ERROR;
    }

  if (createSteeringMethod () != KD_OK)
    {
      hppDout (error, "failed to create steering method");
      return KD_ERROR;
    }
  return KD_OK;
}

ktStatus
TestDirectPath::createDevice ()
{
  double footWidth = 0.2;

  // Create empty device and store it in object
  CkppDeviceComponentShPtr kppDevice =
    CkppDeviceComponent::create ("HBB");

  CkitMat4 absPos;
  CkppKCDBoxShPtr polyBox =
    CkppKCDBox::create ("HBB-Box", 0.5420, 0.8682, 1.6909);
  polyBox->diffuseColor (CkppColor (0.2f, 0.2f, 0.6f, 0.4f));
  absPos.translate (0.0423, 0., 0.7670);
  polyBox->setAbsolutePosition (absPos);
  polyBox->makeCollisionItem ();

  // Create ExtraDof
  CkwsDofShPtr kwsExtraDof;
  kwsExtraDof = CkwsDof::create (false);
  // Bound curvature
  kwsExtraDof->bounds (-5., 5.);
  kwsExtraDof->isBounded (true);
  kppDevice->addExtraDof (kwsExtraDof);


  // Build plan root joint
  CkppJointComponentShPtr kppJoint =
    CkppFreeFlyerJointComponent::create ("ROOTJOINT.BB");

  // Bound joint
  // X
  kppJoint->kwsJoint ()->dof (0)->isBounded (true);
  kppJoint->kwsJoint ()->dof (0)->bounds (-10., 10.);

  // Y
  kppJoint->kwsJoint ()->dof (1)->isBounded (true);
  kppJoint->kwsJoint ()->dof (1)->bounds (-10.,10.);

  // Z
  kppJoint->kwsJoint()->dof (2)->isLocked (true);

  // Rx
  kppJoint->kwsJoint ()->dof (3)->isLocked (true);

  // Ry
  kppJoint->kwsJoint ()->dof (4)->isLocked (true);

  // Rz
  kppJoint->kwsJoint ()->dof (5)->isBounded (false);
  kppJoint->kwsJoint ()->dof (5)->bounds (-M_PI, M_PI);

  kppJoint->kwsJoint ()->dof (0)->userWeightScaler (2);
  kppJoint->kwsJoint ()->dof (1)->userWeightScaler (2);
  kppJoint->kwsJoint ()->dof (5)->userWeightScaler (2 * footWidth / 0.854365);

  // Attach the body to the joint.
  kppJoint->addSolidComponentRef (CkppSolidComponentRef::create (polyBox));
  kppJoint->doesDisplayPath (true);

  // Attach the joint to the device.
  kppDevice->rootJointComponent (kppJoint);

  // Create to config for test
  CkwsConfigShPtr startConfig = CkwsConfig::create (kppDevice->kwsDevice ());
  CkwsConfigShPtr endConfig   = CkwsConfig::create (kppDevice->kwsDevice ());

  startConfig->dofValue (0, 0.0);  // Kappa
  startConfig->dofValue (1, 0.0);  // x
  startConfig->dofValue (2, 0.0);  // y
  startConfig->dofValue (6, 0.0);  // theta

  endConfig->dofValue (0, 0.0); // Kappa
  endConfig->dofValue (1, 2.0); // x
  endConfig->dofValue (2, 1.0); // y
  endConfig->dofValue (6, 0.0); // theta

  attDevice = kppDevice;

  return KD_OK;
}

ktStatus
TestDirectPath::createSteeringMethod()
{
  // Create steering method : Flat Interpolation
  CkwsSteeringMethodShPtr kwsSteeringMethod = CkwsSMLinear::create (false);
  if (!kwsSteeringMethod)
    return KD_ERROR;
  attSteeringMethod = kwsSteeringMethod;
  return KD_OK;
}

